from ..base import _call, _get_read_access_token

'''
All api requests under the auth tab in https://developers.themoviedb.org/4/auth

https://developers.themoviedb.org/4/auth/user-authorization-1
'''

def create_request_token(redirect_to=''):
    '''
    This method generates a new request token that you can ask a user to approve.
    This is the first step in getting permission from a user to read and write data on their behalf.

    Note that there is an optional body you can post alongside this request to set a redirect URL or callback that will be executed once a request token has been approved on TMDb.

    required:
    optional: redirect_to
    '''

    payload = {
        'redirect_to': redirect_to
    }

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('POST', f'https://api.themoviedb.org/4/auth/request_token', headers=headers, payload=payload)

def create_access_token(request_token):
    '''
    This method will finish the user authentication flow and issue an official user access token.
    The request token in this request is sent along as part of the POST body.
    You should still use your standard API read access token for authenticating this request.

    required: request_token
    optional:
    '''

    payload = {
        'request_token': request_token
    }

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/4/auth/access_token', headers=headers, payload=payload)

def delete_access_token(access_token):
    '''
    This method gives your users the ability to log out of a session.

    required: access_token
    optional:
    '''

    payload = {
        'access_token': access_token
    }

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('POST', f'https://api.themoviedb.org/4/auth/access_token', headers=headers, payload=payload)



from ..base import _call, _get_read_access_token

'''
All api requests under the credits tab in https://developers.themoviedb.org/3/credits
'''

def details(credit_id):
    '''
    Get a movie or TV credit details by id.

    required: credit_id
    optional:
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/credit/{credit_id}', headers=headers)



from ..base import _call, _get_read_access_token

'''
All api requests under the tv episodes tab in https://developers.themoviedb.org/3/tv-episodes
'''

def details(tv_id, season_number, episode_number, **kwargs):
    '''
    Get the TV episode details by id.
    Supports append_to_response. https://developers.themoviedb.org/3/getting-started/append-to-response

    required: tv_id, season_number, episode_number
    optional: language, append_to_response
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/season/{season_number}/episode/{episode_number}', params=kwargs, headers=headers)

def changes(episode_id, **kwargs):
    '''
    Get the changes for a TV episode. By default only the last 24 hours are returned.
    You can query up to 14 days in a single query by using the start_date and end_date query parameters.

    required: episode_id
    optional: page, start_date, end_date
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/episode/{episode_id}/changes', params=kwargs, headers=headers)

def account_states(tv_id, season_number, episode_number, **kwargs):
    '''
    Get your rating for a episode.

    required: tv_id, season_number, episode_number, session_id or guest_session_id
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/season/{season_number}/episode/{episode_number}/account_states', params=kwargs, headers=headers)

def credits(tv_id, season_number, episode_number, **kwargs):
    '''
    Get the credits (cast, crew and guest stars) for a TV episode.

    required: tv_id, season_number, episode_number
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/season/{season_number}/episode/{episode_number}/credits', params=kwargs, headers=headers)

def external_ids(tv_id, season_number, episode_number, **kwargs):
    '''
    Get the external ids for a TV episode. We currently support the following external sources.

    Media Databases
        IMDB ID
        TVDB ID
        Freebase MID*
        Freebase ID*
        TVRage ID*

    *Defunct or no longer available as a service.

    required: tv_id, season_number, episode_number
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/season/{season_number}/episode/{episode_number}/external_ids', params=kwargs, headers=headers)

def images(tv_id, season_number, episode_number):
    '''
    Get the images that belong to a TV episode.
    Querying images with a language parameter will filter the results.
    If you want to include a fallback language (especially useful for backdrops) you can use the include_image_language parameter.
    This should be a comma seperated value like so: include_image_language=en,null.

    required: tv_id, season_number, episode_number
    optional:
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/season/{season_number}/episode/{episode_number}/images', headers=headers)

def translations(tv_id, season_number, episode_number):
    '''
    Get the videos that have been added to a TV episode.

    required: tv_id, season_number, episode_number
    optional:
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/season/{season_number}/episode/{episode_number}/translations', headers=headers)

def rate_episode(tv_id, season_number, episode_number, payload, **kwargs):
    '''
    Rate a TV episode.

    A valid session or guest session ID is required. You can read more about how this works https://developers.themoviedb.org/3/authentication/how-do-i-generate-a-session-id.

    required: tv_id, season_number, episode_number, session_id or guest_session_id
    optional:
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('POST', f'https://api.themoviedb.org/3/tv/{tv_id}/season/{season_number}/episode/{episode_number}/rating', params=kwargs, headers=headers, payload=payload)

def delete_rating(tv_id, season_number, episode_number, **kwargs):
    '''
    Remove your rating for a TV episode.

    A valid session or guest session ID is required. You can read more about how this works https://developers.themoviedb.org/3/authentication/how-do-i-generate-a-session-id.

    required: tv_id, season_number, episode_number, session_id or guest_session_id
    optional:
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('POST', f'https://api.themoviedb.org/3/tv/{tv_id}/season/{season_number}/episode/{episode_number}/rating', params=kwargs, headers=headers)

def videos(tv_id, season_number, episode_number, **kwargs):
    '''
    Get the videos that have been added to a TV episode.

    required: tv_id, season_number, episode_number
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/season/{season_number}/episode/{episode_number}/videos', params=kwargs, headers=headers)



from ..base import _call, _get_read_access_token

'''
All api requests under the account tab in https://developers.themoviedb.org/3/account
'''

def details(session_id):
    '''
    Get your account details.

    required: session_id 
    optional:
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/account?session_id={session_id}', headers=headers)

def created_lists(account_id, **kwargs):
    '''
    Get all of the lists created by an account. Will include private lists if you are the owner.

    required: session_id, account_id
    optional: page, language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/account/{account_id}/lists', params=kwargs, headers=headers)

def favorite_movies(account_id, **kwargs):
    '''
    Get the list of your favorite movies.

    required: session_id, account_id
    optional: page, language, sort_by
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/account/{account_id}/favorite/movies', params=kwargs, headers=headers)

def favorite_tv_shows(account_id, **kwargs):
    '''
    Get the list of your favorite TV shows.

    required: session_id, account_id
    optional: page, language, sort_by
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/account/{account_id}/favorite/tv', params=kwargs, headers=headers)

def mark_as_favorite(account_id, session_id, media_type, media_id):
    '''
    This method allows you to mark a movie or TV show as a favorite item.

    required: session_id, account_id, media_type, media_id
    optional:
    '''

    payload = {
        'media_type': media_type,
        'media_id': media_id,
        'favorite': True
        }

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('POST', f'https://api.themoviedb.org/3/account/{account_id}/favorite?session_id={session_id}', headers=headers, payload=payload)

def rated_movies(account_id, **kwargs):
    '''
    Get a list of all the movies you have rated.

    required: session_id, account_id
    optional: page, language, sort_by
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/account/{account_id}/rated/movies', params=kwargs, headers=headers)

def rated_tv(account_id, **kwargs):
    '''
    Get a list of all the TV shows you have rated.

    required: session_id, account_id
    optional: page, language, sort_by
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/account/{account_id}/rated/tv', params=kwargs, headers=headers)

def rated_tv_episodes(account_id, **kwargs):
    '''
    Get a list of all the TV episodes you have rated.

    required: session_id, account_id
    optional: page, language, sort_by
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/account/{account_id}/rated/tv/episodes', params=kwargs, headers=headers)

def movie_watchlist(account_id, **kwargs):
    '''
    Get a list of all the movies you have added to your watchlist.

    required: session_id, account_id
    optional: page, language, sort_by
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/account/{account_id}/watchlist/movies', params=kwargs, headers=headers)

def tv_show_watchlist(account_id, **kwargs):
    '''
    Get a list of all the TV shows you have added to your watchlist.

    required: session_id, account_id
    optional: page, language, sort_by
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/account/{account_id}/watchlist/tv', params=kwargs, headers=headers)

def add_to_watchlist(account_id, session_id, media_type, media_id):
    '''
    Add a movie or TV show to your watchlist.

    required: session_id, account_id, media_type, media_id
    optional:
    '''

    payload = {
        'media_type': media_type,
        'media_id': media_id,
        'watchlist': True
        }

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('POST', f'https://api.themoviedb.org/3/account/{account_id}/favorite?session_id={session_id}', headers=headers, payload=payload)



from ..base import _call, _get_read_access_token

'''
All api requests under the keywords tab in https://developers.themoviedb.org/3/keywords
'''

def details(keyword_id):
    '''
    Gets name and id of keyword

    required: keyword_id 
    optional:
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/keyword/{keyword_id}', headers=headers)

def movies(keyword_id, **kwargs):
    '''
    Get the movies that belong to a keyword.
    We highly recommend using https://developers.themoviedb.org/3/discover/movie-discover instead of this method as it is much more flexible.

    required: keyword_id 
    optional:   language, include_adult
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/keyword/{keyword_id}/movies', params=kwargs, headers=headers)



from ..base import _call, _get_read_access_token

'''
All api requests under the genres tab in https://developers.themoviedb.org/3/genres
'''

def movie(**kwargs):
    '''
    Get the list of official genres for movies.

    required: 
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/genre/movie/list', params=kwargs, headers=headers)

def tv(**kwargs):
    '''
    Get the list of official genres for TV shows.

    required: 
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/genre/tv/list', params=kwargs, headers=headers)



from ..base import _call, _get_read_access_token

'''
All api requests under the people tab in https://developers.themoviedb.org/3/people
'''

def details(person_id, **kwargs):
    '''
    Get the primary person details by id.
    Supports append_to_response. https://developers.themoviedb.org/3/getting-started/append-to-response

    required: person_id
    optional: language, append_to_response
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/person/{person_id}', params=kwargs, headers=headers)

def changes(person_id, **kwargs):
    '''
    Get the changes for a person. By default only the last 24 hours are returned.
    You can query up to 14 days in a single query by using the start_date and end_date query parameters.

    required: person_id
    optional: page, start_date, end_date
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/person/{person_id}/changes', params=kwargs, headers=headers)

def movie_credits(person_id, **kwargs):
    '''
    Get the movie credits for a person.

    required: person_id
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/person/{person_id}/movie_credits', params=kwargs, headers=headers)

def combined_credits(person_id, **kwargs):
    '''
    Get the movie and TV credits together in a single response.

    required: person_id
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/person/{person_id}/combined_credits', params=kwargs, headers=headers)

def external_ids(person_id, **kwargs):
    '''
    Get the external ids for a person. We currently support the following external sources.
    External Sources
    IMDB ID
    Facebook
    Freebase MID
    Freebase ID
    Instagram
    TVRage ID
    Twitter

    required: person_id
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/person/{person_id}/external_ids', params=kwargs, headers=headers)

def images(person_id):
    '''
    Get the images for a person.

    required: person_id
    optional:
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/person/{person_id}/images', headers=headers)

def details(person_id, **kwargs):
    '''
    Get the primary person details by id.
    Supports append_to_response. https://developers.themoviedb.org/3/getting-started/append-to-response

    required: person_id
    optional: language, append_to_response
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/person/{person_id}', params=kwargs, headers=headers)

def tagged_images(person_id, **kwargs):
    '''
    Get the images that this person has been tagged in.

    required: person_id
    optional: language, page
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/person/{person_id}/tagged_images', params=kwargs, headers=headers)

def translations(person_id, **kwargs):
    '''
    Get a list of translations that have been created for a person.

    required: person_id
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/person/{person_id}/translations', params=kwargs, headers=headers)

def latest(**kwargs):
    '''
    Get the most newly created person. This is a live response and will continuously change.

    required:
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/person/latest', params=kwargs, headers=headers)

def popular(**kwargs):
    '''
    Get the list of popular people on TMDb. This list updates daily.

    required:
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/person/popular', params=kwargs, headers=headers)



from ..base import _call, _get_read_access_token

'''
All api requests under the companies tab in https://developers.themoviedb.org/3/companies
'''

def details(company_id):
    '''
    Get a companies details by id.

    required: company_id 
    optional:
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/company/{company_id}', headers=headers)

def alternative_names(company_id):
    '''
    Get the alternative names of a company.

    required: company_id 
    optional:
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/company/{company_id}/alternative_names', headers=headers)

def images(company_id):
    '''
    Get a companies logos by id.

    There are two image formats that are supported for companies, PNG's and SVG's.
    You can see which type the original file is by looking at the file_type field.
    We prefer SVG's as they are resolution independent and as such, the width and height are only there to reflect the original asset that was uploaded.
    An SVG can be scaled properly beyond those dimensions if you call them as a PNG.

    For more information about how SVG's and PNG's can be used, take a read through https://developers.themoviedb.org/3/getting-started/images.

    required: company_id 
    optional:
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/company/{company_id}/images', headers=headers)



from ..base import _call, _get_read_access_token

'''
All api requests under the tv tab in https://developers.themoviedb.org/3/tv
'''

def details(tv_id, **kwargs):
    '''
    Get the primary TV show details by id.
    Supports append_to_response. https://developers.themoviedb.org/3/getting-started/append-to-response

    required: tv_id
    optional: language, append_to_response
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}', params=kwargs, headers=headers)

def account_states(tv_id, **kwargs):
    '''
    Grab the following account states for a session:
        TV show rating
        If it belongs to your watchlist
        If it belongs to your favourite list

    required: tv_id, session_id or guest_session_id
    optional: 
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/account_states', params=kwargs, headers=headers)

def alternative_titles(tv_id, **kwargs):
    '''
    Returns all of the alternative titles for a TV show.

    required: tv_id
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/alternative_titles', params=kwargs, headers=headers)

def changes(tv_id, **kwargs):
    '''
    Get the changes for a TV show. By default only the last 24 hours are returned.
    You can query up to 14 days in a single query by using the start_date and end_date query parameters.

    TV show changes are different than movie changes in that there are some edits on seasons and episodes that will create a change entry at the show level.
    These can be found under the season and episode keys. These keys will contain a series_id and episode_id.
    You can use the https://developers.themoviedb.org/3/tv-seasons/get-tv-season-changes and https://developers.themoviedb.org/3/tv-episodes/get-tv-episode-changes methods to look these up individually.

    required: tv_id
    optional: page, start_date, end_date
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/changes', params=kwargs, headers=headers)

def content_ratings(tv_id, **kwargs):
    '''
    Get the list of content ratings (certifications) that have been added to a TV show.

    required: tv_id
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/content_ratings', params=kwargs, headers=headers)

def credits(tv_id, **kwargs):
    '''
    Get the credits (cast and crew) that have been added to a TV show.

    required: tv_id
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/credits', params=kwargs, headers=headers)

def episode_groups(tv_id, **kwargs):
    '''
    Get all of the episode groups that have been created for a TV show.
    With a group ID you can call the https://developers.themoviedb.org/3/tv-episode-groups/get-tv-episode-group-details method.

    required: tv_id
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/episode_groups', params=kwargs, headers=headers)

def external_ids(tv_id, **kwargs):
    '''
    Get the external ids for a TV show. We currently support the following external sources.

    Media Databases
        IMDb ID
        TVDB ID
        Freebase MID*
        Freebase ID*
        TVRage ID*
    Social IDs
        Facebook
        Instagram
        Twitter

    *Defunct or no longer available as a service.

    required: tv_id
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/external_ids', params=kwargs, headers=headers)

def images(tv_id, **kwargs):
    '''
    Get the images that belong to a TV show.
    Querying images with a language parameter will filter the results.
    If you want to include a fallback language (especially useful for backdrops) you can use the include_image_language parameter.
    This should be a comma seperated value like so: include_image_language=en,null.

    required: tv_id
    optional: language, include_image_language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/images', params=kwargs, headers=headers)

def keywords(tv_id):
    '''
    Get the keywords that have been added to a TV show.

    required: tv_id
    optional: 
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/keywords', headers=headers)

def recommendations(tv_id, **kwargs):
    '''
    Get the list of TV show recommendations for this item.

    required: tv_id
    optional: page, language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/recommendations', params=kwargs, headers=headers)

def reviews(tv_id, **kwargs):
    '''
    Get the reviews for a TV show.

    required: tv_id
    optional: page, language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/reviews', params=kwargs, headers=headers)

def screened_theatrically(tv_id):
    '''
    Get a list of seasons or episodes that have been screened in a film festival or theatre.

    required: tv_id
    optional: 
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/screened_theatrically', headers=headers)

def similar(tv_id, **kwargs):
    '''
    Get a list of similar TV shows.
    These items are assembled by looking at keywords and genres.

    required: tv_id
    optional: page, language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/similar', params=kwargs, headers=headers)

def translations(tv_id, **kwargs):
    '''
    Get a list of the translations that exist for a TV show.

    required: tv_id
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/translations', params=kwargs, headers=headers)

def videos(tv_id, **kwargs):
    '''
    Get the videos that have been added to a TV show.

    required: tv_id
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/videos', params=kwargs, headers=headers)

# def lists(tv_id, **kwargs): should this be here?
#     '''
#     Get a list of lists that this movie belongs to.

#     required: tv_id
#     optional: page, language
#     '''

#     headers = {
#         'authorization': f'Bearer {_get_read_access_token()}',
#         'content-type': 'application/json;charset=utf-8'
#         }

#     return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/lists', params=kwargs, headers=headers)

def latest(**kwargs):
    '''
    Get the most newly created TV show. This is a live response and will continuously change.

    required: 
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/latest', params=kwargs, headers=headers)

def airing_today(**kwargs):
    '''
    Get a list of TV shows that are airing today. This query is purely day based as we do not currently support airing times.
    You can specify a timezone to offset the day calculation. Without a specified timezone, this query defaults to EST (Eastern Time UTC-05:00).

    required: 
    optional: page, language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/airing_today', params=kwargs, headers=headers)

def on_the_air(**kwargs):
    '''
    Get a list of shows that are currently on the air.
    This query looks for any TV show that has an episode with an air date in the next 7 days.

    required: 
    optional: page, language, region
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/on_the_air', params=kwargs, headers=headers)

def popular(**kwargs):
    '''
    Get a list of the current popular TV shows on TMDb. This list updates daily.

    required: 
    optional: page, language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/popular', params=kwargs, headers=headers)

def top_rated(**kwargs):
    '''
    Get a list of the top rated TV shows on TMDb.

    required: 
    optional: page, language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/top_rated', params=kwargs, headers=headers)

def rate_movie(tv_id, rating, **kwargs):
    '''
    Rate a TV show.
    A valid session or guest session ID is required. You can read more about how this works: https://developers.themoviedb.org/3/authentication/how-do-i-generate-a-session-id

    required: tv_id, rating, session_id or guest_session_id
    optional: 
    '''

    payload = {
        'value': rating
        }

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('POST', f'https://api.themoviedb.org/3/tv/{tv_id}/rating', params=kwargs, headers=headers, payload=payload)

def delete_rating(tv_id, **kwargs):
    '''
    Remove your rating for a TV show.
    A valid session or guest session ID is required. You can read more about how this works: https://developers.themoviedb.org/3/authentication/how-do-i-generate-a-session-id

    required: tv_id, session_id or guest_session_id
    optional: 
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('DELETE', f'https://api.themoviedb.org/3/tv/{tv_id}/rating', params=kwargs, headers=headers)


from ..base import _call, _get_read_access_token

'''
All api requests under the lists tab in https://developers.themoviedb.org/3/lists
'''

def details(list_id, **kwargs):
    '''
    Get the details of a list.

    required: list_id
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/list/{list_id}', params=kwargs, headers=headers)

def check_item_status(list_id, **kwargs):
    '''
    You can use this method to check if a movie has already been added to the list.

    required: list_id, movie_id
    optional:
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/list/{list_id}/item_status', params=kwargs, headers=headers)

def create_list(session_id, name, description='', language='en'):
    '''
    Create a list.

    required: session_id, name
    optional: description, language
    '''
    payload = {
        'name': name,
        'description': description,
        'language': language
        }

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('POST', f'https://api.themoviedb.org/3/list?session_id={session_id}', headers=headers, payload=payload)

def add_movie(list_id, session_id, media_id):
    '''
    Add a movie to a list.

    required: list_id, session_id, media_id
    optional: 
    '''
    payload = {
        'media_id': media_id
        }

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('POST', f'https://api.themoviedb.org/3/list/{list_id}/add_item?session_id={session_id}', headers=headers, payload=payload)

def remove_movie(list_id, session_id, media_id):
    '''
    Remove a movie to a list.

    required: list_id, session_id, media_id
    optional: 
    '''
    payload = {
        'media_id': media_id
        }

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('POST', f'https://api.themoviedb.org/3/list/{list_id}/remove_item?session_id={session_id}', headers=headers, payload=payload)

def clear_list(list_id, session_id, confirm="true"):
    '''
    Clear a list.

    required: list_id, session_id, confirm
    optional: 
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('POST', f'https://api.themoviedb.org/3/list/{list_id}/clear?session_id={session_id}&confirm={confirm}', headers=headers)

def delete_list(list_id, session_id):
    '''
    Delete a list.

    required: list_id, session_id
    optional: 
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('DELETE', f'https://api.themoviedb.org/3/list/{list_id}?session_id={session_id}', headers=headers)



import os
from setuptools import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()



setup(
    name='fulltmdb',
    version='0.0.2',
    author='Jacob Hale',
    author_email='jacobhale.hale@gmail.com',
    description=('A wrapper for The Movie Database API v3 and v4.'),
    keywords=['tmdb', 'api-wrapper'],
    url='https://github.com/Jacob212/fulltmdb',
    packages=['fulltmdb','fulltmdb.v3','fulltmdb.v4'],
    test_suite='test',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',
    install_requires=['requests', 'requests_cache'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Utilities',
    ],
)



from ..base import _call, _get_read_access_token

'''
All api requests under the account tab in https://developers.themoviedb.org/4/account
'''

def lists(account_id, **kwargs):
    '''
    Get all of the lists you've created.

    required: account_id 
    optional: page
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/4/account/{account_id}/lists', params=kwargs, headers=headers)

def favorite_movies(account_id, **kwargs):
    '''
    Get the list of movies you have marked as a favorite.

    required: account_id
    optional: page, sort_by
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/4/account/{account_id}/movie/favorites', params=kwargs, headers=headers)

def favorite_tv(account_id, **kwargs):
    '''
    Get the list of TV shows you have marked as a favorite.

    required: account_id
    optional: page, sort_by
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/4/account/{account_id}/tv/favorites', params=kwargs, headers=headers)

def movie_recommendations(account_id, **kwargs):
    '''
    Get a list of your personal movie recommendations.

    required: account_id
    optional: page, sort_by
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/4/account/{account_id}/movie/recommendations', params=kwargs, headers=headers)

def tv_recommendations(account_id, **kwargs):
    '''
    Get a list of your personal TV show recommendations.

    required: account_id
    optional: page, sort_by
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/4/account/{account_id}/tv/recommendations', params=kwargs, headers=headers)

def movie_watchlist(account_id, **kwargs):
    '''
    Get the list of movies you have added to your watchlist.

    required: account_id
    optional: page, sort_by
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/4/account/{account_id}/movie/watchlist', params=kwargs, headers=headers)

def tv_watchlist(account_id, **kwargs):
    '''
    Get the list of TV shows you have added to your watchlist.

    required: account_id
    optional: page, sort_by
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/4/account/{account_id}/tv/watchlist', params=kwargs, headers=headers)

def rated_movies(account_id, **kwargs):
    '''
    Get the list of movies you have rated.

    required: account_id
    optional: page, sort_by
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/4/account/{account_id}/movie/rated', params=kwargs, headers=headers)

def rated_tv(account_id, **kwargs):
    '''
    Get the list of TV shows you have rated.

    required: account_id
    optional: page, sort_by
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/4/account/{account_id}/tv/rated', params=kwargs, headers=headers)



from ..base import _call, _get_read_access_token

'''
All api requests under the tv episode groups tab in https://developers.themoviedb.org/3/tv-episode-groups
'''

def details(group_id, **kwargs):
    '''
    Get the details of a TV episode group. Groups support 7 different types which are enumerated as the following:

        Original air date
        Absolute
        DVD
        Digital
        Story arc
        Production
        TV

    required: group_id
    optional: language
    '''

    headers = {
        'authorization': f'Bearer {_get_read_access_token()}',
        'content-type': 'application/json;charset=utf-8'
        }

    return _call('GET', f'https://api.themoviedb.org/3/tv/episode_group/{group_id}', params=kwargs, headers=headers)



from time import time, sleep
from os import environ
from requests import request
import requests_cache

environ['remaining'] = '40'

class Setup():
    def set_read_access_token(access_token):
        environ['TMDB_READ_ACCESS_TOKEN'] = access_token

    def set_cache(expire_after):
        requests_cache.install_cache(cache_name='TMDB-cache', backend='sqlite', expire_after=expire_after)

    def purge_cache():
        requests_cache.remove_expired_responses()

def _get_read_access_token():
    return environ.get('TMDB_READ_ACCESS_TOKEN')

def _check_status(result):
    if 'success' in result and result['success'] is False:
        raise Exception(result['status_message'])
    return result

def _call(request_type, url, headers=None, params=None, payload=None, disable_cache=None):
    if disable_cache:
        with requests_cache.disabled():
            req = request(request_type, url, params=params, data=payload, headers=headers)
    else:
        req = request(request_type, url, params=params, data=payload, headers=headers)
    headers = req.headers
    if 'X-RateLimit-Remaining' in headers:
        environ['remaining'] = headers['X-RateLimit-Remaining']

    if 'X-RateLimit-Reset' in headers:
        environ['reset'] = headers['X-RateLimit-Reset']

    if int(environ.get('remaining')) < 1:
        current_time = int(time())
        sleep_time = int(environ.get('reset')) - current_time
        print('Rate limit reached. Sleeping for: %d' % sleep_time)
        sleep(abs(sleep_time))
        _call(request_type, url, headers, payload)

    json = req.json()

    return _check_status(json)
